@@ -39,6 +39,13 @@ from hypothesis.strategies._internal.ipaddress import (
 from hypothesis.strategies._internal.lazy import unwrap_strategies
 from hypothesis.strategies._internal.strategies import OneOfStrategy
 
+UnionType: typing.Any
+try:
+    # The type of PEP-604 unions (`int | str`), added in Python 3.10
+    from types import UnionType
+except ImportError:
+    UnionType = ()
+
 try:
     import typing_extensions
 except ImportError:
@@ -59,7 +66,7 @@ except ImportError:
 
 TypeAliasTypes: tuple = ()
 try:
-    TypeAliasTypes += (typing.TypeAlias,)  # type: ignore
+    TypeAliasTypes += (typing.TypeAlias,)
 except AttributeError:
     pass  # Is missing for `python<3.10`
 try:
@@ -75,7 +82,7 @@ except AttributeError:  # pragma: no cover
 
 FinalTypes: tuple = ()
 try:
-    FinalTypes += (typing.Final,)  # type: ignore
+    FinalTypes += (typing.Final,)
 except AttributeError:  # pragma: no cover
     pass  # Is missing for `python<3.8`
 try:
@@ -83,6 +90,59 @@ try:
 except AttributeError:  # pragma: no cover
     pass  # `typing_extensions` might not be installed
 
+ConcatenateTypes: tuple = ()
+try:
+    ConcatenateTypes += (typing.Concatenate,)
+except AttributeError:  # pragma: no cover
+    pass  # Is missing for `python<3.8`
+try:
+    ConcatenateTypes += (typing_extensions.Concatenate,)
+except AttributeError:  # pragma: no cover
+    pass  # `typing_extensions` might not be installed
+
+ParamSpecTypes: tuple = ()
+try:
+    ParamSpecTypes += (typing.ParamSpec,)
+except AttributeError:  # pragma: no cover
+    pass  # Is missing for `python<3.8`
+try:
+    ParamSpecTypes += (typing_extensions.ParamSpec,)
+except AttributeError:  # pragma: no cover
+    pass  # `typing_extensions` might not be installed
+
+TypeGuardTypes: tuple = ()
+try:
+    TypeGuardTypes += (typing.TypeGuard,)
+except AttributeError:  # pragma: no cover
+    pass  # Is missing for `python<3.8`
+try:
+    TypeGuardTypes += (typing_extensions.TypeGuard,)
+except AttributeError:  # pragma: no cover
+    pass  # `typing_extensions` might not be installed
+
+
+RequiredTypes: tuple = ()
+try:
+    RequiredTypes += (typing.Required,)  # type: ignore
+except AttributeError:  # pragma: no cover
+    pass  # Is missing for `python<3.11`
+try:
+    RequiredTypes += (typing_extensions.Required,)
+except AttributeError:  # pragma: no cover
+    pass  # `typing_extensions` might not be installed
+
+
+NotRequiredTypes: tuple = ()
+try:
+    NotRequiredTypes += (typing.NotRequired,)  # type: ignore
+except AttributeError:  # pragma: no cover
+    pass  # Is missing for `python<3.11`
+try:
+    NotRequiredTypes += (typing_extensions.NotRequired,)
+except AttributeError:  # pragma: no cover
+    pass  # `typing_extensions` might not be installed
+
+
 # We use this variable to be sure that we are working with a type from `typing`:
 typing_root_type = (typing._Final, typing._GenericAlias)  # type: ignore
 
@@ -91,13 +151,34 @@ typing_root_type = (typing._Final, typing._GenericAlias)  # type: ignore
 # and are just added for more fancy type annotations.
 # `Final` is a great example: it just indicates
 # that this value can't be reassigned.
-NON_RUNTIME_TYPES = frozenset(
-    (
-        *ClassVarTypes,
-        *TypeAliasTypes,
-        *FinalTypes,
-    )
+NON_RUNTIME_TYPES = (
+    typing.Any,
+    *ClassVarTypes,
+    *TypeAliasTypes,
+    *FinalTypes,
+    *ConcatenateTypes,
+    *ParamSpecTypes,
+    *TypeGuardTypes,
 )
+for name in (
+    "Annotated",
+    "NoReturn",
+    "Self",
+    "Required",
+    "NotRequired",
+    "Never",
+    "TypeVarTuple",
+    "Unpack",
+    "LiteralString",
+):
+    try:
+        NON_RUNTIME_TYPES += (getattr(typing, name),)
+    except AttributeError:
+        pass
+    try:
+        NON_RUNTIME_TYPES += (getattr(typing_extensions, name),)
+    except AttributeError:  # pragma: no cover
+        pass  # typing_extensions might not be installed
 
 
 def type_sorting_key(t):
@@ -170,6 +251,14 @@ def is_a_new_type(thing):
     return isinstance(thing, typing.NewType)  # pragma: no cover  # on 3.8, anyway
 
 
+def is_a_union(thing):
+    """Return True if thing is a typing.Union or types.UnionType (in py310)."""
+    return (
+        isinstance(thing, UnionType)
+        or getattr(thing, "__origin__", None) is typing.Union
+    )
+
+
 def is_a_type(thing):
     """Return True if thing is a type or a generic type like thing."""
     return isinstance(thing, type) or is_generic_type(thing) or is_a_new_type(thing)
@@ -247,9 +336,8 @@ def _try_import_forward_ref(thing, bound):  # pragma: no cover
 
 
 def from_typing_type(thing):
-    # We start with special-case support for Union and Tuple - the latter
-    # isn't actually a generic type. Then we handle Literal since it doesn't
-    # support `isinstance`.
+    # We start with special-case support for Tuple, which isn't actually a generic
+    # type; then Final, Literal, and Annotated since they don't support `isinstance`.
     #
     # We then explicitly error on non-Generic types, which don't carry enough
     # information to sensibly resolve to strategies at runtime.
@@ -332,7 +420,7 @@ def from_typing_type(thing):
         # if there is more than one allowed type, and the element type is
         # not either `int` or a Union with `int` as one of its elements.
         elem_type = (getattr(thing, "__args__", None) or ["not int"])[0]
-        if getattr(elem_type, "__origin__", None) is typing.Union:
+        if is_a_union(elem_type):
             union_elems = elem_type.__args__
         else:
             union_elems = ()
@@ -577,7 +665,7 @@ _global_type_lookup.update(
     }
 )
 if hasattr(typing, "SupportsIndex"):  # pragma: no branch  # new in Python 3.8
-    _global_type_lookup[typing.SupportsIndex] = st.integers() | st.booleans()  # type: ignore
+    _global_type_lookup[typing.SupportsIndex] = st.integers() | st.booleans()
 
 
 def register(type_, fallback=None, *, module=typing):
@@ -613,7 +701,7 @@ def resolve_Type(thing):
         # This branch is for Python < 3.8, when __args__ was not always tracked
         return st.just(type)  # pragma: no cover
     args = (thing.__args__[0],)
-    if getattr(args[0], "__origin__", None) is typing.Union:
+    if is_a_union(args[0]):
         args = args[0].__args__
     # Duplicate check from from_type here - only paying when needed.
     args = list(args)
@@ -782,13 +870,33 @@ def resolve_Callable(thing):
     # use of keyword arguments and we'd rather not force positional-only.
     if not thing.__args__:  # pragma: no cover  # varies by minor version
         return st.functions()
+
+    *args_types, return_type = thing.__args__
+
     # Note that a list can only appear in __args__ under Python 3.9 with the
     # collections.abc version; see https://bugs.python.org/issue42195
+    if len(args_types) == 1 and isinstance(args_types[0], list):
+        args_types = tuple(args_types[0])
+
+    pep612 = ConcatenateTypes + ParamSpecTypes
+    for arg in args_types:
+        # awkward dance because you can't use Concatenate in isistance or issubclass
+        if getattr(arg, "__origin__", arg) in pep612 or type(arg) in pep612:
+            raise InvalidArgument(
+                "Hypothesis can't yet construct a strategy for instances of a "
+                f"Callable type parametrized by {arg!r}.  Consider using an "
+                "explicit strategy, or opening an issue."
+            )
+    if getattr(return_type, "__origin__", None) in TypeGuardTypes:
+        raise InvalidArgument(
+            "Hypothesis cannot yet construct a strategy for callables which "
+            f"are PEP-647 TypeGuards (got {return_type!r}).  "
+            "Consider using an explicit strategy, or opening an issue."
+        )
+
     return st.functions(
-        like=(lambda: None)
-        if len(thing.__args__) == 1 or thing.__args__[0] == []
-        else (lambda *a, **k: None),
-        returns=st.from_type(thing.__args__[-1]),
+        like=(lambda *a, **k: None) if args_types else (lambda: None),
+        returns=st.from_type(return_type),
     )
 
 
