@@ -19,6 +19,7 @@ import re
 import sys
 import types
 from functools import wraps
+from keyword import iskeyword
 from tokenize import detect_encoding
 from types import ModuleType
 from typing import TYPE_CHECKING, Callable
@@ -84,7 +85,19 @@ def function_digest(function):
     return hasher.digest()
 
 
-def get_signature(target):
+def check_signature(sig: inspect.Signature) -> None:
+    # Backport from Python 3.11; see https://github.com/python/cpython/pull/92065
+    for p in sig.parameters.values():
+        if iskeyword(p.name) and p.kind is not p.POSITIONAL_ONLY:
+            raise ValueError(
+                f"Signature {sig!r} contains a parameter named {p.name!r}, "
+                f"but this is a SyntaxError because `{p.name}` is a keyword. "
+                "You, or a library you use, must have manually created an "
+                "invalid signature - this will be an error in Python 3.11+"
+            )
+
+
+def get_signature(target, *, follow_wrapped=True):
     # Special case for use of `@unittest.mock.patch` decorator, mimicking the
     # behaviour of getfullargspec instead of reporting unusable arguments.
     patches = getattr(target, "patchings", None)
@@ -97,6 +110,7 @@ def get_signature(target):
     if isinstance(getattr(target, "__signature__", None), inspect.Signature):
         # This special case covers unusual codegen like Pydantic models
         sig = target.__signature__
+        check_signature(sig)
         # And *this* much more complicated block ignores the `self` argument
         # if that's been (incorrectly) included in the custom signature.
         if sig.parameters and (inspect.isclass(target) or inspect.ismethod(target)):
@@ -116,10 +130,13 @@ def get_signature(target):
 
         if is_generic_type(target):
             sig = inspect.signature(target.__init__)
+            check_signature(sig)
             return sig.replace(
                 parameters=[v for k, v in sig.parameters.items() if k != "self"]
             )
-    return inspect.signature(target)
+    sig = inspect.signature(target, follow_wrapped=follow_wrapped)
+    check_signature(sig)
+    return sig
 
 
 def arg_is_required(param):
@@ -431,7 +448,7 @@ def nicerepr(v):
         return re.sub(r"(\[)~([A-Z][a-z]*\])", r"\g<1>\g<2>", pretty(v))
 
 
-def arg_string(f, args, kwargs, reorder=True):
+def repr_call(f, args, kwargs, reorder=True):
     if reorder:
         args, kwargs = convert_positional_arguments(f, args, kwargs)
 
@@ -444,7 +461,10 @@ def arg_string(f, args, kwargs, reorder=True):
         for a in sorted(kwargs):
             bits.append(f"{a}={nicerepr(kwargs[a])}")
 
-    return ", ".join(bits)
+    rep = nicerepr(f)
+    if rep.startswith("lambda") and ":" in rep:
+        rep = f"({rep})"
+    return rep + "(" + ", ".join(bits) + ")"
 
 
 def check_valid_identifier(identifier):
@@ -482,6 +502,8 @@ def accept({funcname}):
 def define_function_signature(name, docstring, argspec):
     """A decorator which sets the name, argspec and docstring of the function
     passed into it."""
+    if name == "<lambda>":
+        name = "_lambda_"
     check_valid_identifier(name)
     for a in argspec.args:
         check_valid_identifier(a)
@@ -605,7 +627,7 @@ def define_function_signature_from_signature(name, docstring, signature):
     ]
 
     def accept(f):
-        fsig = inspect.signature(f)
+        fsig = inspect.signature(f, follow_wrapped=False)
         must_pass_as_kwargs = []
         invocation_parts = []
         for p in pos_args:
@@ -687,7 +709,7 @@ def proxies(target: "T") -> Callable[[Callable], "T"]:
     replace_sig = define_function_signature_from_signature(
         target.__name__.replace("<lambda>", "_lambda_"),  # type: ignore
         target.__doc__,
-        get_signature(target),
+        get_signature(target, follow_wrapped=False),
     )
 
     def accept(proxy):
